def application(environ, start_response):
    start_response('200 OK', [('Content-type', 'text/html'),
                              ('test-header', 'TEST!')])
    return ['test1']

                   
